// GCDThread.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001

   CHANGE: 11/6/01
     modified pullAnchorText() to get to the second occurrence
     of "detail.lasso" before looking for the anchor
*/
/* 
  This thread sends a query to GCD at
  http://213.203.29.50/action.lasso, and pulls out the matching
  results. They are written to the display table as SearchResult objects.

  Sample query for "x-men 1":
    "http://213.203.29.50/action.lasso?Bk_Name=x-men&Issue=
       1&-Max=50&-find=Search&-db=Issues&-lay=Overview&-format=results2.html"

  Meaning of fields
     "Bk_Name=" : title name  (with -op=equals& in front)
     "Issue"    : issue       (with -op=equals& in front)
     "-Max=50"  : max number of results
     "-db=Issue" : use Issue database
     "-lay=Overview" : display the results in overview format
     "-format=results2.html"
*/

import java.io.*;
import java.net.*;
import java.util.*;
import COM.stevesoft.pat.*; // Regular Expression


public class GCDThread extends Thread
{
  private String title, issue, enIDPass;
  private ComicSearch searchIntf;

  public GCDThread(String Ctitle, String Cissue, String pass, ComicSearch cs)
  {
    title = Ctitle;
    issue = Cissue;
    enIDPass = pass;
	searchIntf = cs;
  }

  public void run()
  {
    try {
      String GCDkey = "?-op=equals&Bk_Name=" + URLEncoder.encode(title) + 
	                  "&-op=equals&Issue=" + issue +
                      "&-Max=50&-find=Search&-db=Issues&-lay=Overview&" +
					  "&-format=results2.html";				   
      URL u = new URL ( "http://213.203.29.50/action.lasso" + GCDkey );
      URLConnection conn = u.openConnection();   // send query to GCD

      if (enIDPass != null)    // Send authorization though the proxy server
        conn.setRequestProperty("Proxy-Authorization", "Basic " + enIDPass);
      conn.setDoOutput( true );

      // read results
      BufferedReader dis = new BufferedReader ( 
	                new InputStreamReader ( conn.getInputStream() ) );
      String line, tempdata = "";
	  SearchResults sr;
      Regex trMatch = new Regex("</tr>");     // for delimiting the text searches

      while ( ((line=dis.readLine())!=null) && 
	           !searchIntf.stopSearchPressed() ) {
        tempdata += line;
        trMatch.search(line);
        if (trMatch.didMatch()) {
          sr = GCDFilter(tempdata);    // filter text once a </tr> tag is found
          if (sr != null && sr.hasValues())
           searchIntf.writeRow(sr);    // write the results to the table
           tempdata = "";        // clear tempdata for next search
        }
      }
    } 	
	catch ( IOException e ) {
      System.out.println("IO Exception: " + e);
    }
    searchIntf.threadFinished("GCD");
  } // end of run()


  SearchResults GCDFilter(String htmldata)
  // Filter information from the text, and search for the title.
  // If the title is found then return a SearchResults object.
  // There is no need to search for the number due to the accuracy
  // of the original query.
  {
    String s = cleanMatch(htmldata);  // remove some tags

//    System.out.println("GCDFilter 1: " + s);
    Regex comics = new Regex("(?i)" + title);    // ignore case
    comics.search(s);      // search for the title
    if (comics.didMatch()) {
      String URLText = pullURL(s);
	  String anchorText = pullAnchorText(s);
      // System.out.println("GCD URLText: " + URLText);
	  // System.out.println("GCD anchorText: " + anchorText);
	  return new SearchResults( "http://213.203.29.50/" + URLText, 
	                       anchorText + " " + issue, "GCD" );    
    }     
    return null;
  }


  String cleanMatch(String s)
  // remove several tags
  {
    s = cutTag(s, "<b>");  s = cutTag(s, "<B>");
    s = cutTag(s, "</b>"); s = cutTag(s, "</B>");
    s = cutTag(s, "<br>"); s = cutTag(s, "</BR>");
    s = cutTag(s, "<p>");  s = cutTag(s, "<P>");
    s = cutTag(s, "</p>"); s = cutTag(s, "</P>");
    return s;
  }


  String cutTag(String s, String tag)
  // Replace the tags in s with ""'s
  {
    Regex line = new Regex(tag, "");
    return line.replaceAll("" + s);   // so not null string
  }


  String pullURL(String s)
  // find URL which begin with "details.lasso" and ends with "\">"
  // Output: "details.lasso?uniqueid=MRVXMN0Y010"
  {
    Regex tag = new Regex("(?i)details.lasso");
    Regex end = new Regex("\">");
    tag.search( "" + s );     // so not null string
    end.search( tag.stringMatched() + tag.right() );

    if (end.didMatch())
      return end.left();
    return "missing url";
  }


  // Filter anchor text
  // The anchor text starts after the ">", and ends with the
  // "<" of </a>
  // "<a href=\"details.lasso?uniqueid=MRVXMN0Y010\">
  //         <font size=\"1\" face=\"Verdana,Arial,Helvetica\">X-Men</a>"
  // output : X-Men

  // Change 11/6/01: GCD now has two rows containing 'details.lasso";
  // must go to the second one to get the anchor
  String pullAnchorText (String s)
  {
    Regex begin = new Regex("(?i)details.lasso");
    Regex end = new Regex("</a>");
    Regex anch = new Regex(">");

    begin.search( s );        // Find 1st "details.lasso"
    if (begin.didMatch()) {
      String restS = begin.right();
      begin.search( restS );   // find 2nd "details.lasso"
      if (begin.didMatch()) {
        // Find </a> from ?uniqueid=MRVXMN0Y010"><font size="1"
        // face="Verdana,Arial,Helvetica">X-Men</a>
        end.search( begin.right() );
        if (end.didMatch()) {
          // Find ">" from ?uniqueid=MRVXMN0Y010"><font size="1"
          // face="Verdana,Arial,Helvetica">X-Men
          anch.search( end.left() );
          if (anch.didMatch()) {
            anch.search(anch.right());
            return anch.right();      // return "X-men"
          } 
        } 
      }
    } 
    return "Anchor not found";
  } // end of pullAnchorText()

} // end of GCDThread class
