
// SourceMap.java

// Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001

// This file contains the SourceMap and SourceInfo classes

/*
  A SourceMap object contains details about the info. sources
  used by ComicSearch. The map is a collection of
  source IDs, each mapped to a SourceInfo object containing
  a source name and URL.

  Two uses:
   1. The "Sources" combo box is a list of source IDs; when
      a user clicks on an ID, a dialog box shows the corresponding
	  source name and URL.

   2. The source IDs in the table can be clicked upon. When a 
      source ID is clicked, the corresponding source URL is loaded 
	  into the browser.

*/

import java.io.*;
import java.util.*;


public class SourceMap
{
  private HashMap hm;   // for source Id /sourceInfo values

  public SourceMap()
  // initilise with the basic maps; the others are added when
  // the Titles database is read in
  {
    hm = new HashMap();    
    hm.put("AH", new SourceInfo("Auction Hawk",
                   "http://www.auctionhawk.com"));
    hm.put("GCD", new SourceInfo("The Grand Comic Database",
                   "http://213.203.29.50"));
    hm.put("GG", new SourceInfo("Google",
                   "http://www.google.com"));
    hm.put("SS", new SourceInfo(
             "Nick Simon's Silver Age Marvel Comics Cover Index",
             "http://www.geocities.com/Area51/Zone/4414/index.html"));
  }


  public void add(String sID, String sName, String sURL)
  // for adding further source infomation
  {
    if (!hm.containsKey(sID))
      hm.put(sID, new SourceInfo(sName, sURL));
  }

  public String getSourceName(String sID)
  {
    if (hm.containsKey(sID)) {
      SourceInfo si = (SourceInfo) hm.get(sID);
      return si.getSourceName();
    }
    return null;
  }


  public String getSourceURL(String sID)
  {
    if (hm.containsKey(sID)) {
      SourceInfo si = (SourceInfo) hm.get(sID);
      return si.getSourceURL();
    }
    return null;
  }

  
  public Object[] getSourceIDs()
  // used to initilise the "Sources" combo box
  {
    Object[] o = hm.keySet().toArray();
    Arrays.sort( o );   // so in alphabetical order
    return o;
  }

}  // end of SourceMap class



// **************************************************************

class SourceInfo
{
  private String sourceName;
  private String sourceURL;

  public SourceInfo(String n, String u)
  {
    sourceName = n;
    sourceURL = u;
  }

  public String getSourceName()
  {  return sourceName;  }

  public String getSourceURL()
  {  return sourceURL;  }

}  // end of SourceInfo class
